"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debugLib = require("debug");
const path = require("path");
const webpack = require("webpack");
const winston = require("winston");
const debug = debugLib("azure-functions-pack:WebpackRunner");
class WebpackRunner {
    static run(options) {
        options.indexFileName = options.indexFileName || "index.js";
        options.packHostIndexFileName = options.packHostIndexFileName || "index.gen.js";
        options.outputPath = options.outputPath || ".funcpack";
        options.uglify = options.uglify || false;
        options.watch = options.watch || false;
        options.ignoredModules = options.ignoredModules || [];
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            debug("Setting up paths");
            const indexPath = path.join(options.projectRootPath, options.outputPath, options.indexFileName);
            const packHostPath = path.join(options.projectRootPath, options.outputPath, options.packHostIndexFileName);
            const ignoredModules = {};
            for (const mod of options.ignoredModules) {
                ignoredModules[mod.toLowerCase()] = mod;
            }
            debug("Creating Webpack Configuration");
            let config = {
                entry: packHostPath,
                externals: ignoredModules,
                node: {
                    __dirname: false,
                    __filename: false,
                },
                output: {
                    filename: options.indexFileName,
                    library: "index",
                    libraryTarget: "commonjs2",
                    path: path.join(options.projectRootPath, options.outputPath),
                },
                plugins: [],
                target: "node",
                watch: options.watch,
            };
            if (options.uglify) {
                debug("Adding uglify plugin");
                try {
                    config.plugins.push(new webpack.optimize.UglifyJsPlugin());
                }
                catch (e) {
                    debug(e);
                }
            }
            try {
                if (options.editConfig) {
                    const customizeFunctionPath = path.join(options.projectRootPath, options.editConfig);
                    const customizeFunction = require(customizeFunctionPath);
                    config = customizeFunction(config, webpack);
                }
            }
            catch (e) {
                winston.error(e);
                throw new Error("Could not apply customize function");
            }
            debug("Creating Webpack instance");
            const compiler = webpack(config);
            debug("Started webpack");
            if (options.watch) {
                compiler.watch(null, (err, stats) => __awaiter(this, void 0, void 0, function* () {
                    debug("Webpack recompile");
                    if (err || stats.hasErrors()) {
                        return reject(err || stats.toString({ errors: true }));
                    }
                    debug("\n" + stats.toString());
                }));
            }
            else {
                compiler.run((err, stats) => __awaiter(this, void 0, void 0, function* () {
                    debug("Webpack finished");
                    if (err || stats.hasErrors()) {
                        return reject(err || stats.toString({ errors: true }));
                    }
                    debug("\n" + stats.toString());
                    resolve();
                }));
            }
        }));
    }
}
exports.WebpackRunner = WebpackRunner;
//# sourceMappingURL=webpack-runner.js.map